<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Modules\Cadastros\Services\gql00400TipoDeRiscoService;
use Zeedhi\Framework\DTO\Response\Message;

class gql00400TipoDeRiscoController extends \Zeedhi\Framework\Controller\Report
{
    const TIPO = array
    (
        "Fornecedor" => "F",
        "Cliente"    => "C",
        "Prospect"   => "P",
        "Fabricante" => "B",
    );

    protected $connection;
    protected $qr2Strategy;
    
    public function __construct 
    (
        reportService $reportService,
        gql00400TipoDeRiscoService $service,
        $connection,
        $qr2Strategy
    ) {
        parent::__construct($reportService);
        $this->connection    = $connection;
        $this->service       = $service;
        $this->qr2Strategy   = $qr2Strategy;
    }
    
    public function insertTipoRisco(DTO\Request $request, DTO\Response $response) {
		try {
            $row = $request->getRow(); 
            
            $codrisco = $this->service->getCodRisco($row);
            if ($codrisco) {
            	throw new \Exception("CODIGO_CADASTRADO");	
            }
            
            $nmrisco = $this->service->getNmRisco($row);
            if ($nmrisco) {
            	throw new \Exception("DESC_CADASTRADA");	
            }
            
            if ((empty($codrisco)) and (empty($nmrisco))) {
    			$this->service->insertTipoRisco($row);
            }
            
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function confirmarEditTpRisco(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow(); 

            $nmrisco = $this->service->getNmRisco($row);
            
            if ($nmrisco) {
            	throw new \Exception("DESC_CADASTRADA");	
            }
            
			$this->service->updateTipoRisco($row);
		} 
		catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
    }
    
    public function deleteTpRisco(DTO\Request $request, DTO\Response $response) {
		try {
            $row = $request->getRow();
            
            $ocorrencia = $this->service->getOcorrencia($row);
            if (!empty($ocorrencia)) {
            	throw new \Exception("DEL_NOT_ALOWED_OCORRENCIA");	
            }
            
            $toleconf = $this->service->getToleconf($row);
            if (!empty($toleconf)) {
            	throw new \Exception("DEL_NOT_ALOWED_TOLECONF");
            }
            
			$this->service->deleteTpRisco($row);
			
			$response->addMessage(new Message("Alterado com sucesso."));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
}