<?php

/* Registro10GRRF
 * @package    Logic
 * @subpackage Registro/Grrf
 * @author     francismararaujo - 22/02/2013
 */
class Registro10GRRF extends RegistroGRRF{

    const REGISTRO_10 = '10';

    //@todo Verificar se estas constantes podem ir para nivel superior.
    const SIMPLES_NAO_OPTANTE = parent::CARACTER_UM;
    const SIMPLES_OPTANTE = parent::CARACTER_DOIS;

    const CARACTER_HIFEN = ' - ';

    private $_estrutura;
    private $_endereco;
    private $_nrInscricao;
    private $_tpInscricao;
    private $_nrRecolhimento;
    private $_fpas;
    private $_simples;
    private $_codCentralizadorFGTS;
    private $_CNPJCentralizador;

    /* Preparacao para a criacao do Registro 10.
     * @param EstruturamEntity $estrutura   Instancia de EstruturamEntity.
     * @param EnderecoEntity   $endereco    Instancia de EnderecoEntity.
     * @return void
     * @author francismararaujo - 22/02/2013
     */

    public function geraRegistro10(EstruturamEntity $estrutura, EnderecoEntity $endereco){
        $this->_estrutura = $estrutura;
        $this->_endereco = $endereco;
        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_fpas = $this->_estrutura->historico->getCdfpas();
        $this->_simples = $this->_estrutura->historico->getIdoptsimples();
        $this->_geraRegistro();
    }

    /* Cria o Registro 10.
     * @return void
     * @author francismararaujo - 22/02/2013
     */

    private  function _geraRegistro(){

        //bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_10, $this->_tpInscricao, $this->_nrInscricao);

        //bloco 4 | 018-053 | preencher com zeros.
        $this->atribuiDado(parent::CARACTER_VAZIO, 36, parent::CARACTER_ZERO);

        //bloco 5 | 054-093 | razao Social.
        $this->_atribuiRazaoSocial($this->_estrutura->historico->getNmrazsocestrut());

        //bloco 6 | 094-143 | endereco.
        $endereco = $this->_endereco->getDsendereco();
        $this->atribuiDado($endereco, 50, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 7 | 144-163 | bairro.
        $bairro = $this->_endereco->bairro;
        $this->atribuiDado($bairro, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 8 | 164-171 | cep.
        //@todo Diferentes de 20000000, 30000000, 70000000 ou 80000000.
        $cep = $this->_endereco->getNrcependereco();
        $this->atribuiDado($cep, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 9 | 172-191 | cidade.
        $cidade = $this->_endereco->municipio;
        $this->atribuiDado($cidade, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 10 | 192-193 | unidade da federacao (estado).
        $estado = $this->_endereco->getSgestado();
        $this->atribuiDado($estado, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 11 | 194-205 | telefone.
        $telefone = $this->_endereco->ddd.$this->_endereco->telefone;
        $this->atribuiDado($telefone, 12, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 12 | 206-212 | cnae.
        $this->atribuiDado(
            $this->_estrutura->historico->getCdcnae(), 7, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA
        );

        //bloco 13 | 213 | simples.
        $this->_atribuiSimples($this->_estrutura->historico->getIdoptsimples());

        //bloco 14 | 214-216 | fpas.
        $this->_atribuiFpas($this->_estrutura->historico->getCdfpas());

        //bloco 15 | 217-359 | preencher com brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 143, parent::CARACTER_BRANCO);

        //bloco 16 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);

    }

    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }

    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }

    public function setCNPJCentralizador($CNPJCentralizador)
    {
        $this->_CNPJCentralizador = $CNPJCentralizador;
    }

    public function setCodCentralizadorFGTS($codCentralizadorFGTS)
    {
        $this->_codCentralizadorFGTS = $codCentralizadorFGTS;
    }

    private function _atribuiRazaoSocial($razaosocial){
        if ($razaosocial) {
            $razaosocialtradado = str_replace(self::CARACTER_HIFEN, parent::CARACTER_BRANCO, $razaosocial);
        }else{
            $razaosocialtradado =  parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($razaosocialtradado, 40, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

    }

    /* Atribui aliquota RAT.
     * @param string $rat Aliquota RAT.
     * @return void
     * @author francismararaujo - 25/02/2013 - Implementacao de validacao do RAT.
     */

    private function _atribuiAliquotaRAT($rat){
        //@todo verificar layout e arquivo ARTE para validacoes.

        $codRecol = array(
            '145', '307', '317', '327', '337', '345', '640', '660',
        );

        $fpas = array(
            '604', '647', '825', '833', '868',
        );

        // Nao pode ser informado p/ FPAS 604 com recolhimento de cod. 150 em compet. posteriores a 10/2001.
        // Nao pode ser informado para os codigos de recolhimento 145, 307, 317, 327, 337, 345, 640 e 660.
        if ((($this->_fpas === '604'
            && $this->_nrRecolhimento === '150')
            || (array_search($this->_nrRecolhimento, $codRecol) !== false))
        ) {
            $rat = parent::CARACTER_VAZIO;
        } else {
            // Sera zeros p/ FPAS 604, 647, 825, 833 e 868 (emp. domestico) e para a empresa optante pelo SIMPLES.
            if (((array_search($this->_fpas, $fpas) !== false)
                ||($this->_simples === self::SIMPLES_OPTANTE))
            ) {
                $rat = parent::CARACTER_ZERO;
            }
        }

        // 1 casa para inteiro e 1 casa para decimal - Ex. 3.0% => 30.
        $this->atribuiDado($rat, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    /* Atribui codigo centralizador.
     * @param boolean $centraliza 		 Flag identificador para centralizar recolhimento.
     * @param string  $cnpjCentralizador CNPJ do centralizador.
     * @return void
     * @author alessandrobartels - 02/06/2011
     * 		   thalesvalias 	 - 31/01/2012 - Implementacao de validacao do codigo centralizador.
     *         alessandrobartels - 11/06/2012 - Criado codCentralizadorFGTS como atributo da classe.
     */

    private function _atribuiCodigoCentralizador(){

        $codRecol = array(
            '130', '135', '150', '155', '211', '317', '337', '608',
        );

        // Para os codigos de recolhimento 130, 135, 150, 155, 211, 317, 337, 608 o codigo centralizador sera 0.
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false) || ($this->_fpas === '868')) {
            $this->_codCentralizadorFGTS = parent::CARACTER_ZERO;
        } else {
            if ($this->_codCentralizadorFGTS === null) {
                $this->_codCentralizadorFGTS = parent::CARACTER_ZERO;
            }
        }

        $this->atribuiDado($this->_codCentralizadorFGTS, 1);
    }

    /* Atribui opcao pelo SIMPLES.
     * @param string $optanteSimples Opcao pelo SIMPLES.
     * @return void
     * @author alessandrobartels - 02/06/2011
     * 		   thalesvalias      - 31/01/2012 - Implementacao de validacao do SIMPLES.
     */

    private function _atribuiSimples($optanteSimples){
        $fpas = array(
            '582', '639', '663', '671', '680', '736', '868',
        );

        // Para os fpas 582, 639, 663, 671, 680, 736, 868 nao optante pelo simples.
        if ((array_search($this->_fpas, $fpas) !== false)) {
            $simples = self::SIMPLES_NAO_OPTANTE;
        } else {
            if ($this->_nrRecolhimento === '640') {
                $simples = parent::CARACTER_BRANCO;
            } else {
                if ($optanteSimples === parent::SIM) {
                    $simples = self::SIMPLES_OPTANTE;
                } else {
                    $simples = self::SIMPLES_NAO_OPTANTE;
                }
            }
        }

        $this->atribuiDado($simples, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    /* Atribui codigo FPAS valido.
     * @param string $fpas Codigo FPAS.
     * @return void
     * @author alessandrobartels - 02/06/2011
     */

    private function _atribuiFpas($fpas){
        $this->atribuiDado($fpas, 3, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }

    /* Atribui codigo de terceiros.
     * @param string $codigoTerceiros Codigo de terceiros.
     * @return void
     * @author alessandrobartels - 02/06/2011
     */

    private function _atribuiCodigoTerceiros($codigoTerceiros){
        /* . N�o pode ser informado para os c�digos de recolhimento 145, 307, 317, 327, 337, 345, 640 e 660.
           � Deve ficar em branco quando o SIMPLES for igual a 2 , 3 ou 6.
           � Deve ser preenchido com zeros para o FPAS 582 quando a compet�ncia for maior ou igual a 10/1998.
           � Deve ser preenchido com zeros para FPAS 868 ( empregador dom�stico).
           � Deve estar contido na tabela de terceiros, inclusive zeros se SIMPLES for igual a 1, 4 ou 5.
           � Sempre que n�o informado o campo deve ficar em branco.
         */
        $codRecol = array(
            '145', '307', '317', '327', '337', '345', '640', '660',
        );

        $fpas = array(
            '582', '868',
        );

        if ((array_search($this->_nrRecolhimento, $codRecol) !== false)
            ||($this->_simples === self::SIMPLES_OPTANTE)
        ) {
            $codigoTerceiros = parent::CARACTER_BRANCO;
        } else {
            if (array_search($this->_fpas, $fpas) !== false) {
                $codigoTerceiros = parent::CARACTER_ZERO;
            }
        }

        $this->atribuiDado($codigoTerceiros, 4, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    /* Atribui codigo da GPS.
     * @param string $codigoGPS Codigo da GPS.
     * @return void
     * @author alessandrobartels - 02/06/2011
     */

    private function _atribuiCodigoGPS($codigoGPS){
        /* @todo verificar layout e arquivo ARTE para validacoes. Atentar no caso do codigo vir nulo.*/

        $codRecol = array(
            '115', '150', '211', '650',
        );

        if (array_search($this->_nrRecolhimento, $codRecol) === false) {
            $codigoGPS = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($codigoGPS, 4, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    /* Atribui percentual de filantropia.
     * @param string $percentualFilantropia Percentual de Filantropia.
     * @return void
     * @author alessandrobartels - 02/06/2011
     */

    private function _atribuiPercentualFilantropia($percentualFilantropia){
        if ($this->_fpas !== '639') {
            $percentualFilantropia = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($percentualFilantropia, 5, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    /* Atribui valor do Salario Familia.
     * @param float $vrSalarioFamilia Valor do salario familia.
     * @return void
     * @author thalesvalias - 31/01/2012
     */

    private function _atribuiSalarioFamilia($vrSalarioFamilia) {

        $codRecol = array(
            '145', '307', '327', '345', '640', '650', '660',
        );

        // @todo Verificar leiaute - validar para a competencia 13.
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false) || ($this->_fpas === '868')) {
            $vrSalarioFamilia = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($vrSalarioFamilia, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }

    /* Atribui valor do Salario Maternidade.
     * @param float $vrSalarioMaternidade Valor do salario Maternidade.
     * @return void
     * @author thalesvalias - 31/01/2012
     */

    private function _atribuiSalarioMaternidade($vrSalarioMaternidade) {

        $codRecol = array(
            '130', '135', '145', '211', '307', '317', '327', '337', '345', '640', '650', '660',
        );

        // @todo Verificar leiaute - validar para a competencia 13.
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false) || ($this->_fpas === '868')) {
            $vrSalarioMaternidade = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($vrSalarioMaternidade, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    public function alteraDadoRegistro10($registroMag, $dado) {
        //$registro, $dado, $posicaoInicial, $tamanhoCampo, $caracter, $orientacao

        $codRecol = array(
            '150', '155', '608', '907', '908', '910',
        );

        if ((array_search($this->_nrRecolhimento, $codRecol) === false)){
            // Salario Familia - Posicao Inicial = 235 Tamanho = 15. Subtrai 1 da posicao inicial, pois o array inicia em 0.
            $linha = $this->alteraDado(
                $registroMag->getDslinharegistro(), $dado[self::GRRF_SALARIO_FAMILIA], (235 - 1), 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA
            );
            $registroMag->setDslinharegistro($linha);
        }

        // Salario Maternidade - Posicao Inicial = 250 Tamanho = 15. Subtrai 1 da posicao inicial, pois o array inicia em 0.
        $linha = $this->alteraDado(
            $registroMag->getDslinharegistro(), $dado[self::GRRF_SALARIO_MATERNIDADE], (250 - 1), 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA
        );
        $registroMag->setDslinharegistro($linha);

        $registroMag->setVrsalfamilia($dado[self::GRRF_SALARIO_FAMILIA]);
        $registroMag->setVrsalmaternidad($dado[self::GRRF_SALARIO_MATERNIDADE]);

    }

}